local SILO_PROTOCOL = "silo_v1"

local class = require("silo.lib.class")
local Messaging = class("silo.Messaging")

function Messaging:new(ctx)
  self.listeners = {}
  self.seq = 0

  ctx.bus:register("event.rednet_message", function(sender, message, protocol)
    if protocol == SILO_PROTOCOL then
      self.seq = math.max(self.seq, message.seq + 1)

      for id, listener in pairs(self.listeners) do
        local function unregister()
          self.listeners[id] = nil
        end

        listener(sender, message, unregister)
      end
    end
  end)
end

function Messaging:listen(listener)
  table.insert(self.listeners, listener)
end

function Messaging:populateSequenceData(message)
  message.seq = self.seq
  self.seq = self.seq + 1
end

function Messaging:send(message, callback)
  self:populateSequenceData(message)
  rednet.broadcast(message, SILO_PROTOCOL)

  if callback ~= nil then
    self:listen(function(sender, recv, unregister)
      if recv.resp == message.seq then
        callback(sender, recv)
        unregister()
      end
    end)
  end
end

function Messaging:sendDirect(id, message, callback)
  self:populateSequenceData(message)
  rednet.send(id, message, SILO_PROTOCOL)

  if callback ~= nil then
    self:listen(function(sender, recv, unregister)
      if recv.resp == message.seq then
        callback(sender, recv)
        unregister()
      end
    end)
  end
end

function Messaging:open()
  for _, side in pairs(redstone.getSides()) do
    if peripheral.getType(side) == "modem" then
      rednet.open(side)
    end
  end
end

function Messaging:close()
  for _, side in pairs(redstone.getSides()) do
    if peripheral.getType(side) == "modem" then
      rednet.close(side)
    end
  end
end

return Messaging
